#'ProxiesAttractor
#'
#'@author Carmen Alvarez-Castro, \email{carmen.alvarez-castro@cmcc.it}
#'@description This function computes two dinamical proxies of the attractor: The local dimension (d) and the inverse of the persistence (theta). 
#'These two parameters will be used as a condition for the bias correction function DynBiasCorrection  
#'Funtion based on the matlab code (@author Davide Faranda, \email{davide.faranda@lsce.ipsl.fr}) used in "Faranda, D., Alvarez-Castro, M.C., Messori, G., Rodriguez, D., and Yiou, P. (2019). 
#'The hammam effect or how a warm ocean enhances large scale atmospheric predictability. Nature Communications, 10(1), 1316. DOI = https://doi.org/10.1038/s41467-019-09305-8 " 
#'
#'@param dat data to create the attractor. Must be a matrix with the timesteps in rows and gridpoints in columns -> dat(time,grids)
# 
#'@param quanti threshold for the computation of the attractor
#'
#'@param iplot to have a quick view of the proxies and the attractor
#'
#'@return dim and theta
#'
#'@return A list of 2:
#'\itemize{
#'\item\code{$dim} {A vector with the same length than the timesteps: nrow(dat)}
#'\item\code{$theta} {A vector with the same length than the timesteps: nrow(dat)}}
#'@references 
#' \url{https://doi.org/10.1038/s41467-019-09305-8}
#' 
#'@import s2dverification
#'@import fields
#'@import pracma
#'
#'@examples
#' Creating an example of matrix dat(time,grids):
tm=24*6 # time
gm=5000 # grids
m=matrix(rand(1,tm*gm),nrow=tm,ncol=gm) 
qm=0.90 # imposing a threshold
#'# selecting the option to create a plot to see the attractor 'iplot=TRUE'
#' example 
ExampleAttractor=ProxiesAttractor(dat=m,quanti=qm,iplot=TRUE)
#'
ProxiesAttractor <- function(dat,quanti,iplot=FALSE){
  npoints=nrow(dat)
  dim=numeric(npoints)
  theta=numeric(npoints)
  time=seq(1:npoints)
  l=1
  for (l in 1:npoints){
    #computation of the distances from the iteration l
    distance=pdist2(dat[l,],dat)
    #Taking the logarithm of the distance
    logdista=-log(distance)
    n=1
    for(n in 1:npoints){
      if(logdista[n]=="Inf"){logdista[n]=NaN}
      n=n+1}  
    #Compute the thheshold corresponding to the quantile
    thresh=quantile(logdista, quanti,na.rm=TRUE)
    
    #Computation of the inverse of persistence 'theta'
    Li<-which(logdista>thresh)
    #Length of each cluster
    Ti<-diff(Li)
    N=length(Ti)
    q=1-quanti
    Si=Ti-1
    Nc=length(which(Si>0))
    N=length(Ti)
    theta[l]=(sum(q*Si)+N+Nc-sqrt(((sum(q*Si)+N+Nc)^2)-8*Nc*sum(q*Si)))/(2*sum(q*Si))
    
    #computation of local dimension 'dim'
    #Sort the exceedances  
    logdista=sort(logdista)
    #Find all the Peaks over Thresholds. 
    findidx=which(logdista>thresh)
    logextr=logdista[findidx[[1]]:(length(logdista)-1)]
    #The inverse of the dimension is just the average of the exceedances
    dim[l]=1/mean(logextr-thresh)
    l=l+1
    if(l<npoints){print(paste('processing ',l, ' of ',npoints," timesteps",sep=''))}
    if(l==npoints){print("complete")}
  }
  # plotting dim vs theta
  if(iplot==TRUE){
    layout(matrix(c(1,3,2,3),2,2))
    plot(time,dim,xlab='time',ylab='d',main='local dimension',type='l')
    plot(time,theta,xlab='time',ylab='theta',main='theta') 
    plot(dim,theta,col='blue',main="Proxies of the Attractor",xlab="local dimension",ylab="theta",lwd=8,'p')
  }
  return(list(dim=dim,theta=theta))
}
